"""cascade run deletion

Revision ID: 8f8dba68fd3b
Revises: 567bc23fd1ac
Create Date: 2020-02-10 12:52:49.540462

"""
from alembic import op
from sqlalchemy.engine import reflection

# pylint: disable=no-member
# alembic dynamically populates the alembic.context module

# revision identifiers, used by Alembic.
revision = "8f8dba68fd3b"
down_revision = "567bc23fd1ac"
branch_labels = None
depends_on = None


def upgrade():
    bind = op.get_context().bind
    inspector = reflection.Inspector.from_engine(bind)
    has_tables = inspector.get_table_names()

    if "runs" in has_tables and "run_tags" in has_tables:
        op.drop_constraint("run_tags_run_id_fkey", table_name="run_tags", type="foreignkey")
        op.create_foreign_key(
            "run_tags_run_id_fkey",
            source_table="run_tags",
            referent_table="runs",
            local_cols=["run_id"],
            remote_cols=["run_id"],
            ondelete="CASCADE",
        )


def downgrade():
    bind = op.get_context().bind
    inspector = reflection.Inspector.from_engine(bind)
    has_tables = inspector.get_table_names()

    if "runs" in has_tables and "run_tags" in has_tables:
        op.drop_constraint("run_tags_run_id_fkey", table_name="run_tags", type="foreignkey")
        op.create_foreign_key(
            "run_tags_run_id_fkey",
            source_table="run_tags",
            referent_table="runs",
            local_cols=["run_id"],
            remote_cols=["run_id"],
        )



import pytest
from dagster_postgres.run_storage import PostgresRunStorage
from dagster_postgres.schedule_storage.schedule_storage import PostgresScheduleStorage

from dagster.utils import file_relative_path
from dagster.utils.test.postgres_instance import TestPostgresInstance


@pytest.fixture(scope="function")
def hostname(conn_string):  # pylint: disable=redefined-outer-name, unused-argument
    return TestPostgresInstance.get_hostname()


@pytest.fixture(scope="function")
def conn_string():  # pylint: disable=redefined-outer-name, unused-argument
    with TestPostgresInstance.docker_service_up_or_skip(
        file_relative_path(__file__, "docker-compose.yml"), "test-postgres-db"
    ) as conn_str:
        yield conn_str


@pytest.fixture(scope="function")
def clean_storage(conn_string):  # pylint: disable=redefined-outer-name
    storage = PostgresRunStorage.create_clean_storage(conn_string)
    assert storage
    return storage


@pytest.fixture(scope="function")
def clean_schedule_storage(conn_string):  # pylint: disable=redefined-outer-name
    storage = PostgresScheduleStorage.create_clean_storage(conn_string)
    assert storage
    return storage


@pytest.fixture(scope="module")
def multi_postgres():  # pylint: disable=redefined-outer-name
    compose_file_path = file_relative_path(__file__, "docker-compose-multi.yml")
    with TestPostgresInstance.docker_service_up_or_skip(
        compose_file_path,
        "test-run-storage-db",
        conn_args={
            "port": 5434,
            "hostname": TestPostgresInstance.get_hostname("POSTGRES_TEST_RUN_STORAGE_DB_HOST"),
        },
    ) as run_storage_conn_string:
        with TestPostgresInstance.docker_service_up_or_skip(
            compose_file_path,
            "test-event-log-storage-db",
            conn_args={
                "port": 5433,
                "hostname": TestPostgresInstance.get_hostname(
                    "POSTGRES_TEST_EVENT_LOG_STORAGE_DB_HOST"
                ),
            },
        ) as event_log_storage_conn_string:
            yield (run_storage_conn_string, event_log_storage_conn_string)



import time

import psycopg2

from dagster import Field, IntSource, Selector, StringSource
from dagster.seven import quote_plus as urlquote


def get_conn(conn_string):
    conn = psycopg2.connect(conn_string)
    conn.set_isolation_level(psycopg2.extensions.ISOLATION_LEVEL_AUTOCOMMIT)
    return conn


def pg_config():
    return Selector(
        {
            "postgres_url": str,
            "postgres_db": {
                "username": StringSource,
                "password": StringSource,
                "hostname": StringSource,
                "db_name": StringSource,
                "port": Field(IntSource, is_required=False, default_value=5432),
            },
        }
    )


def pg_url_from_config(config_value):
    if config_value.get("postgres_url"):
        return config_value["postgres_url"]

    return get_conn_string(**config_value["postgres_db"])


def get_conn_string(username, password, hostname, db_name, port="5432"):
    return "postgresql://{username}:{password}@{hostname}:{port}/{db_name}".format(
        username=username,
        password=urlquote(password),
        hostname=hostname,
        db_name=db_name,
        port=port,
    )


def wait_for_connection(conn_string):
    retry_limit = 20

    while retry_limit:
        try:
            psycopg2.connect(conn_string)
            return True
        except psycopg2.OperationalError:
            pass

        time.sleep(0.2)
        retry_limit -= 1

    assert retry_limit == 0
    raise Exception("too many retries for db at {conn_string}".format(conn_string=conn_string))
